unit CubeU;
{
     Delphi 2.0 sample conversion of the 3DR E1SMOOTH example.

     Converted from C by : Roger Rowland
                           Kirton, Suffolk UK
                           Tel : +44 1394 448259

     EMail c/o Contship Containerlines: 100070.2773@compuserve.com

     Freeware - May 1996    }

{ $Z2,Y-,W-,L-,D-,T+,P+ }
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, I3dr3dg, StdCtrls, ExtCtrls;

type
  PointF_t8 = array[0..7] of PointF_t;
  Fixed16_t30 = array[0..29] of Fixed16_t;

  TForm1 = class(TForm)
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure action_trackball(msg: word; x, y: integer);
    procedure AppAction;
    procedure Display;
  end;

var
  Form1     : TForm1;

{ define 8 vertices of a cube }

  cube_p: PointF_t8                 = ((x:-0.5; y:-0.5; z:-0.5),
                                       (x:-0.5; y:-0.5; z: 0.5),
                                       (x:-0.5; y: 0.5; z:-0.5),
                                       (x:-0.5; y: 0.5; z: 0.5),
                                       (x: 0.5; y:-0.5; z:-0.5),
                                       (x: 0.5; y:-0.5; z: 0.5),
                                       (x: 0.5; y: 0.5; z:-0.5),
                                       (x: 0.5; y: 0.5; z: 0.5));

{ define 8 normal vectors to the vertices defined above }

  cube_n: PointF_t8                = ((x:-1.0; y:-1.0; z:-1.0),
                                      (x:-1.0; y:-1.0; z: 1.0),
                                      (x:-1.0; y: 1.0; z:-1.0),
                                      (x:-1.0; y: 1.0; z: 1.0),
                                      (x: 1.0; y:-1.0; z:-1.0),
                                      (x: 1.0; y:-1.0; z: 1.0),
                                      (x: 1.0; y: 1.0; z:-1.0),
                                      (x: 1.0; y: 1.0; z: 1.0));

{ Define index array in the form which describes the faces of a cube

  Each face is represented as
    number_of_vertices_for_that_face, vtx_1_in_cube_p_array, vtx_2_in_cube_p_array, ...

  Note that we enumerate vertices in the index array in CW order }


  cube_v: Fixed16_t30                = ( 4,   0, 2, 3, 1,
                                         4,   4, 5, 7, 6,
                                         4,   1, 3, 7, 5,
                                         4,   0, 4, 6, 2,
                                         4,   3, 2, 6, 7,
                                         4,   0, 1, 5, 4);

{ Define camera location, direction and up-vector }

  camloc : PointFW_t = (x:0.0; y:0.0; z:-6.0; w:1.0);
  camdir : PointF_t  = (x:0.0; y:0.0; z:1.0);
  camup  : PointF_t  = (x:0.0; y:1.0; z:0.0);
  off    : Fixed32_t = 0;

{ This is for the application specific initialization.

  The code in the resize function is necessary for EVERY application
  that uses 3DR/GP, but here we can place the code for fine tuning
  the scene: specific lights or materials or something else }

{ Put an infinite light somewhere on the negative
  z-axis. It will shine white. }

  Lightdir : PointFW_t = (x:0.0; y:0.0; z:-1.0; w:0.0);
  Lightcol : ColorF_t  = (r:1.0; g:1.0; b:1.0);

{ We shall use something that resembles plastic to make the cube }

  Oa       : ColorF_t  = (r:0.15; g:0.03; b:0.12);
  Od       : ColorFA_t = (r:0.7; g:0.12; b:0.4; a:1.0);
  Os       : ColorF_t  = (r:1.0; g:0.1; b:0.1);
  Se       : Float_t   = 100.0;
  cull     : Float_t   = 50.0;
  dimmer   : Float_t   = 1.0;

  hDC3DR    : HDC;

  CWidth    : integer;
  CHeight   : integer;
  hRC       : R3dHandle_t;
  hGC       : G3dHandle_t;
  R, dR     : TransformF_t;
  axis      : PointF_t;
  theta     : Float_t;
  dtheta    : Float_t;
  spinmode  : integer;
  cdisplay  : integer;
  cactive   : boolean;

  center    : PointF_t;      { action_trackball static variables }
  radius    : Float_t;
  down      : integer;
  mat       : TransformF_t;

  z1, z2    : Float_t;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  hDC3DR   := 0;
  hRC      := 0;
  hGC      := nil;

{ initialise control and static variables }

  CWidth   := (512-64);
  CHeight  := (512-64);
  theta    := 0.0;
  dtheta   := 0.0;
  spinmode := 0;
  down     := 0;
  cdisplay := 0;
  cactive  := false;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  CWidth := Form1.width; CHeight := Form1.height;
  if CWidth > Cheight then CWidth := CHeight else CHeight := CWidth;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if hRC <>0 then R3dDeleteRC(hRC);
  if hGC <> nil then G3dDeleteGC(hGC);
  if hDC3DR <> 0 then ReleaseDC(Handle, hDC3DR);
end;

procedure TForm1.action_trackball(msg: word; x, y: integer);
begin
  case msg of
    WM_LBUTTONDOWN:begin
    { User pressed left mouse button, so we must

        - set the corresponding flag
        - define the center of rotation as the center of the window
        - define the rotation radius as the the window half-size }

        down     := 1;
        center.x := (CWidth / 2);
        center.y := (CHeight / 2);
        if CWidth > CHeight then radius:=center.y else radius:=center.x;

    { Then we store current transformation matrix for future use }

        G3dGetTransform( hGC, @mat);
        G3dCopyMatrix( @mat, @R);

    { And call G3dTrackball() with current center, radius and current cursor
      position (in client coordinates) to initialize trackball internal
      parameters. }

    { Note that we must stop spinning at that time. }

        G3dTrackball( TB_START, @center, radius,
                      (x*1.0), ((CHeight-y)*1.0), @axis, @theta);
        spinmode := 0;
    end;

    WM_MOUSEMOVE:begin

    { If the user moves mouse while pressing the left button we must
      rotate our cube according to the mouse movement.

      To do this we shall call G3dTrackball() again and rotate dR matrix over
      the axis and by angle obtained.
      Then we must multiply this matrix by the current transform matrix
      stored in the mat variable above to obtain the new transform matrix R.
      Then we must set the current transform matrix to the R matrix

      Note that we are displaying the scene with every mouse movement. }

        if (down=1) then
        begin
          G3dTrackball( TB_MOVE, @center, radius, (x*1.0),
                        ((CHeight-y)*1.0), @axis, @theta);
          G3dRotateMatrix( theta, axis.x, axis.y, axis.z, @dR);
          G3dMultMatrix( @dR, @mat, @R);
          G3dSetTransform( hGC, @R);
          Display;
	end;
        end;

    WM_LBUTTONUP:begin

    { User released the left mouse button, so we must

        - call G3dTrackball() to determine the final rotation parameters
        - if user rotated the cube by the small angle (from the moment the
          left button was pressed till the moment it was released) we shall
          not spin the cube. }

        if (down=1) then
        begin
          G3dTrackball( TB_END, @center, radius, (x*1.0),
                        ((CHeight-y)*1.0), @axis, @dtheta);
          theta := 0.0;
          if ( dtheta > 0.017) then spinmode := 1
          else
          begin
            spinmode := 0;
            dtheta := 0.0;
          end;
          Display;
        end;
        down := 0;
        end;
  end;
end;

procedure TForm1.Display;
var
  mat   : TransformF_t;
begin
  G3dSetWindowSize( hGC, (CWidth*1.0), (CHeight*1.0));
  G3dSetCameraPort( hGC, (CWidth*1.0), 0.0, (CHeight*1.0), 0.0);

  if (spinmode=1) then
  begin

  { If we need to spin the cube we must:

      - increment the rotation angle by the dtheta value which
        we obtained from the G3dTrackball() calls. Note that we
        maintain rotation angle in range from -2*PI to 2*PI 

      - then we rotatate dR matrix which was updated by the 
        trackball during the G3dTrackball calls aover the axis
        defined also by the trackball by the current theta angle.

      - the final transform matrix is defined as a multiply of
        the rotated dR matrix and the R matrix which contains the
        transformation state when the mouse button was released. }

    theta := theta + dtheta;
    if (theta >  3.141593) then theta := theta - 6.2831853;
    if (theta < -3.141593) then theta := theta + 6.2831853;
    G3dRotateMatrix( theta, axis.x, axis.y, axis.z, @dR);
    G3dMultMatrix( @dR, @R, @mat);
    G3dSetTransform( hGC, @mat);
  end;

{ We must clear the back buffers before writing to it }

  R3dClearBuffers( hRC, R3D_CI_DRAW_BUFFER or R3D_CI_ZBUFFER);

{ We use PolygonList representation of a cube, that is why we
  defined cube_v index array. As you know, the cube consists
  of 6 faces and 8 vertices. }

  G3dPolygonListF( hGC, 6, 8, @cube_p, longint(sizeof(PointF_t)),
                   @cube_n, longint(sizeof(PointF_t)), nil,
                   longint(sizeof(PointF_t)), nil,
                   longint(sizeof(ColorFA_t)), @cube_v);

{ We must swap back and fore buffers to show the cube }

  R3dSwapBuffers( hRC, 0);
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
{ mouse button down }
  if Button=mbleft then action_trackball( WM_LBUTTONDOWN, x, y);
end;

procedure TForm1.AppAction;
begin
  while cactive do
  begin
    if cdisplay=1 then Display;
    Application.ProcessMessages;
  end;
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
{ mouse moved }
  action_trackball( WM_MOUSEMOVE, x, y);
end;

procedure TForm1.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
{ mouse button up }
  if Button=mbleft then action_trackball( WM_LBUTTONUP, x, y);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  cactive := false;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  if (hRC = 0) then
  begin
    hDC3DR := GetDC(Handle);
    if (HDC3DR <> 0) then
    begin
      hRC := R3dCreateRC(hDC3DR, R3D_RI_DOUBLE_BUFFER);
      if (hRC <> 0) then
      begin
        R3dSetState( hRC, R3D_STA_ENABLES, R3D_SE_SHADING);
        R3dSetState( hRC, R3D_STA_ENABLES, R3D_SE_Z_BUFFERING);
        R3dSetState( hRC, R3D_STA_DISABLES, R3D_SE_CLIPPING);

      { Because the faces of the cube have CW orientation, we must say that
        if the face has CCW orientation it is not a FRONT one }

        hGC := G3dCreateGC( hRC);
        G3dSetState( hGC, G3DL_FRONT_CCW, @off);

      { Tuning the camera

        Camera projection is set to perspective
        Camera view volume is defined as a cube from -1 to 1 for all the axis.
        Camera port corresponds to the entire window

        Note that near and far are set to 5 and 7 respectively.
        Together with camera location at z=-6 this also gives us view volume
        dimensions over z axis from -6+5=-1 to -6+7=1 }

        G3dSetCameraProjection( hGC, G3DP_PERSPECTIVE);
        G3dSetCameraPosition( hGC, @camloc, @camdir, @camup);
        G3dSetCameraView( hGC, 1.0, -1.0, 1.0, -1.0, 5.0, 7.0);
        G3dSetWindowSize( hGC, (CWidth*1.0), (CHeight*1.0));
        G3dSetCameraPort( hGC, (CWidth*1.0), 0.0, (CHeight*1.0), 0.0);

      { Clear transformation stack &
        Prepare matrices for use with trackball }

        G3dClearStack( hGC);
        G3dIdentityMatrix( @dR);
        G3dIdentityMatrix( @R);

      { Application specific initialisation }

      { Now turn on the light (dimmer=1)
        Put it to the desired location
        And define it's colors }

        G3dSetLight( hGC, 0, G3DL_DIMMER, @dimmer);
        G3dSetLight( hGC, 0, G3DL_LOCATION, @Lightdir);
        G3dSetLight( hGC, 0, G3DL_DIFFUSE, @Lightcol);
        G3dSetLight( hGC, 0, G3DL_SPECULAR, @Lightcol);

      { We shall use material number 0 with our cube }

        G3dSetCurrentMatt( hGC, 0);
        G3dSetMatt( hGC, 0, G3DM_AMBIENT, @Oa);
        G3dSetMatt( hGC, 0, G3DM_DIFFUSE, @Od);
        G3dSetMatt( hGC, 0, G3DM_SPECULAR, @Os);
        G3dSetMatt( hGC, 0, G3DM_SHINE, @Se);
        G3dSetMatt( hGC, 0, G3DM_BACK_CULL, @cull);

      { Set window background to grey }

        R3dSetState( hRC, R3D_STA_BG_COLOR, $ffc0c0c0);

      { And show the window }

        R3dClearBuffers( hRC, R3D_CI_DRAW_BUFFER or R3D_CI_ZBUFFER);
        R3dSwapBuffers( hRC, 0);
        cdisplay := 1;
        cactive := true;
      end;
    end;
  end;
  AppAction;
end;

end.
