unit I3DR3DG;
{
    Delphi 2.0 interface to Intel's 3DR/RE and 3DR/GP. This unit provides
    everything in the 3DR.H and 3DG.H files except the packet options in RE.
    Most of it has been tested.

     Converted from C by : Roger Rowland
                           Kirton, Suffolk UK
                           Tel : +44 1394 448259

     EMail c/o Contship Containerlines: 100070.2773@compuserve.com

     Freeware - May 1996     }

{$A+,B-,F-,G+,I+,K+,N+,P+,T+,V+,X+,D-,L-,Q-,R-,S-,W-,Y-,Z2}

{
   Dictionary of Acronyms, Keywords etc.,

    R3d - 3DR, misspelt, for compilation reasons.

    RC  - Rendering Context, as in RCInfo
    Pt  - Point, as in PtLong_T
    Clr - Color, as in ClrLong_T
    Ptr - Pointer, as in LongPtrToByte_T
    Prm - Primitive, as in R3dPrmType_T
    Vtx - Vertex, as in R3dVtxType_T
    SRC - Source, as in R3D_STATE_SRC_BLEND
    DST - Destination, as in R3D_STATE_DST_BLEND
    ROP - Raster (logical) Operation, as in R3D_STATE_ROP_FUNCTION
    Msg - Message, as in R3dMsgType
    MSG - Message, as in R3D_MSG_NOP
    Pkt - Packet, as in R3dPktType
    PKT - Packet, as in R3D_PKT_END
    Hdr - Header, as in MsgHdrType
    Gen - Generic, as in MsgGenHdr
    Bmp - Bitmap, as in MsgBmpHdr
    Sta - State, as in PktStaType
    STA - STATE, as in R3D_STA_ENABLES
    DB  - DoubleBuffer, as in  R3D_DB_FRONT
    ZF  - Z buffer Function, as in R3D_ZF_NEVER 
    LT  - Less Then, as in R3D_ZF_LT
    EQ  - EQual to, as in R3D_ZF_EQ
    LE  - Less then or Equal to, as in R3D_ZF_LE
    GT  - Greater Then, as in R3D_ZF_GT
    NE  - Not Equal to, as in R3D_ZF_NE
    GE  - Greater then or Equal to, as in R3D_ZF_GE
    BL  - Blending Function, as in R3D_BL_SRC_INV
    INV - INVerse, as in R3D_BL_SRC_INV
    RP  - Raster oPeration function, as in R3D_RP_AND
    TF  - Texture Filter function, as in R3D_TF_MIPMAP
    TM  - Texture Modulation function, as in R3D_TF_MIPMAP
    TT  - Texture Type information , as in R3D_TT_KEEP
    BT  - Bitmap Type information , as in R3D_BT_ALPHA
    ST  - State Type information , as in R3D_ST_LEFT
    CT  - Clearbuffer Type information , as in R3D_CT_DRAWBUFFER

   G3d 	- 3DG reordered for compilation reasons.

   23F	- indicates a 2 by 3 floating point matrix
     A	- indicates alpha blending variable present as in ColorFA
   BMP	- bitmap image file format
 Coords - coordinate system
   Eqns	- equations
   F	- indicates floating point 32 bit version
   FA	- indicates floating point 32 bit with alpha (for colors)
   FW	- indicates floating point 32 bit homogeneous coordinates
   GC	- Geometry Context
  Isect - intersection such as the intersection of a line and a plane.
   L	- indicates Fixed point 32bit version
   LW	- indicates Fixed point 32bit homogeneous coordinates
luBackSub- lower, upper back substitution
luDecomp- lower triangular, upper triangular decomposition of a matrix.
   Matt	- material properties of a surface
   Mult	- multiply
   Norm,nrm- normal vector perpendicular to a surface.
   Pow	- power or exponentiation
   prim	- drawing primitive such as a line or polygon
   prop	- property or attribute such as of a light source
   pt	- Point in some coordinate system
   Q	- quadratic.  2nd degree (3rd order) polynomial
   qd	- quasi definitive, or quadrature, or quick and dirty.
	  Indicates that a function is a quick approximation.
   Sqrt	- square root
   S	- indicates short (16bit) components such as for colors
   SA	- indicates short (16bit) components with alpha
   trans- transform, a 4x4 matrix for transforming points
   vec	- vector, usually x,y,z
   vtx	- vertex such as a polygon vertex
   Win	- window
   W	- indicates the homogeneous coordinate, w value is present as in PointFW_t
}

interface
uses SysUtils, WinProcs;

type
{ common type definitions }
  Byte_t      = Byte;
  Char_t      = Char;
  Short_t     = Shortint;
  Long_t      = LongInt;
  Float_t     = single;
  Fixed16_t   = Word;
  Fixed32_t   = LongInt;
  Word_t      = Word;
  Dword_t     = LongInt;
  LPVoid_t    = Pointer;
  LPByte_t    = ^Byte_t;
  LPChar_t    = ^Char_t;
  LPShort_t   = ^Short_t;
  LPLong_t    = ^Long_t;
  LPFloat_t   = ^Float_t;
  LPFixed16_t = ^Fixed16_t;
  LPFixed32_t = ^Fixed32_t;
  LPWord_t    = ^Word_t;
  LPDword_t   = ^Dword_t;
  PVoid_t     = Pointer;
  PByte_t     = ^Byte_t;
  PChar_t     = ^Char_t;
  PShort_t    = ^Short_t;
  PLong_t     = ^Long_t;
  PFloat_t    = ^Float_t;
  PFixed16_t  = ^Fixed16_t;
  PFixed32_t  = ^Fixed32_t;
  PWord_t     = ^Word_t;
  PDword_t    = ^Dword_t;
  DWProc_t    = function: Dword_t;
  R3dHandle_t = Dword_t;

{ 3DR data types - Points }

  LPR3dFixedVtx_t = ^R3dFixedVtx_t;
  FPR3dFixedVtx_t = ^R3dFixedVtx_t;
  R3dFixedVtx_t   = record
    x, y, z: Fixed32_t;
  end;

  LPR3dFixedVtxClr_t = ^R3dFixedVtxClr_t;
  FPR3dFixedVtxClr_t = ^R3dFixedVtxClr_t;
  R3dFixedVtxClr_t   = record
    x, y, z   : Fixed32_t;
    r, g, b, a: Fixed16_t;
  end;

  LPR3dFixedVtxTex_t = ^R3dFixedVtxTex_t;
  FPR3dFixedVtxTex_t = ^R3dFixedVtxTex_t;
  R3dFixedVtxTex_t   = record
    x, y, z: Fixed32_t;
    q, u, v: Fixed32_t;
  end;

  LPR3dFixedVtxTexClr_t = ^R3dFixedVtxTexClr_t;
  FPR3dFixedVtxTexClr_t = ^R3dFixedVtxTexClr_t;
  R3dFixedVtxTexClr_t   = record
    x, y, z   : Fixed32_t;
    q, u, v   : Fixed32_t;
    r, g, b, a: Fixed16_t;
  end;

  LPR3dFloatVtx_t = ^R3dFloatVtx_t;
  FPR3dFloatVtx_t = ^R3dFloatVtx_t;
  R3dFloatVtx_t   = record
    x, y, z: Float_t;
  end;

  LPR3dFloatVtxClr_t = ^R3dFloatVtxClr_t;
  FPR3dFloatVtxClr_t = ^R3dFloatVtxClr_t;
  R3dFloatVtxClr_t   = record
    x, y, z   : float_t;
    r, g, b, a: float_t;
  end;

  LPR3dFloatVtxTex_t = ^R3dFloatVtxTex_t;
  FPR3dFloatVtxTex_t = ^R3dFloatVtxTex_t;
  R3dFloatVtxTex_t   = record
    x, y, z: Float_t;
    q, u, v: Float_t;
  end;

  LPR3dFloatVtxTexClr_t = ^R3dFloatVtxTexClr_t;
  FPR3dFloatVtxTexClr_t = ^R3dFloatVtxTexClr_t;
  R3dFloatVtxTexClr_t   = record
    x, y, z   : Float_t;
    q, u, v   : Float_t;
    r, g, b, a: Float_t;
  end;

{ 3DR data types - Message Header }

  LPR3dMsgHdr_t = ^R3dMsgHdr_t;
  FPR3dMsgHdr_t = ^R3dMsgHdr_t;
  R3dMsgHdr_t   = record
    MsgHdrType   : Word_t;
    MsgHdrVersion: Word_t;
    MsgHdrSize   : Dword_t;
    MsgHdrRC     : R3dHandle_t;
  end;

{ 3DR data types - Generic Message definition }

  LPR3dMsgGen_t = ^R3dMsgGen_t;
  FPR3dMsgGen_t = ^R3dMsgGen_t;
  R3dMsgGen_t   = record
    MsgGenHdr   : R3dMsgHdr_t;
    MsgGenInfo0 : Word_t;
    MsgGenInfo1 : Word_t;
    MsgGenData  : Dword_t;                 { Buffer-wide Modifier }
    MsgGenBuffer: Array[0..0] of Dword_t;  { variable size data   }
  end;

{ 3DR data types - Texture Message definition }

  LPR3dMsgTex_t = ^R3dMsgTex_t;
  FPR3dMsgTex_t = ^R3dMsgTex_t;
  R3dMsgTex_t   = record
    MsgTexHdr       : R3dMsgHdr_t;
    MsgTexInfo      : Word_t;
    MsgTexBitsPixel : Word_t;
    MsgTexMipLevel  : Word_t; 
    MsgTexWidth     : Word_t;
    MsgTexRows      : Word_t;   
    MsgTexWidthBytes: Short_t;
    MsgTexId        : Dword_t;
    MsgTexData      : Dword_t;                  { Buffer-wide Modifier }
    MsgTexBuffer    : Array[0..0] of  Dword_t;  { variable size data   }
  end;

{ 3DR data types - Bitmap Message definition }

  LPR3dMsgBmp_t = ^R3dMsgBmp_t;
  FPR3dMsgBmp_t = ^R3dMsgBmp_t;
  R3dMsgBmp_t   = record
    MsgBmpHdr       : R3dMsgHdr_t;
    MsgBmpPad       : Word_t;
    MsgBmpInfo      : Word_t;
    MsgBmpBitsPixel : Word_t; 
    MsgBmpWidth     : Word_t;
    MsgBmpHeight    : Word_t; 
    MsgBmpWidthBytes: Short_t; 
    MsgBmpSrcX      : Short_t;
    MsgBmpSrcY      : Short_t;
    MsgBmpDstX      : Short_t;
    MsgBmpDstY      : Short_t;
    MsgBmpData      : Dword_t;                 { Buffer-wide Modifier }
    MsgBmpBuffer    : Array[0..0] of Dword_t;  { variable size data   }
  end;

{ 3DG type definitions }

  G3dHandle_t       = pointer;

  G3dContextProps_t = (G3D_NO_DRAW_ORDER);

  G3dState_t = (G3DL_FACETED_NORMALS,
		G3DL_FLAT_SHADING,
		G3DL_TEXTURE_MOD,
		G3DL_FRONT_CCW,
		G3DL_NO_CLIPPING,
                G3DL_LIGHTING,
                G3DL_PRM_TYPE);

  G3dCoords_t = (G3DC_OBJECT_COORDS,
		 G3DC_WINDOW_COORDS);

  G3dCapabilities_t = record
    zbufferPlanes : Fixed32_t;
  end;

{ ----------------------COLORS------------------------ }

  LPColorF_t = pointer; {^ColorF_t;}
  ColorF_t = record
    r,g,b : Float_t;
  end;

  LPColorFA_t = pointer; {^ColorFA_t;}
  ColorFA_t = record
    r,g,b,a : Float_t;
  end;

  LPColorSA_t = pointer; {^ColorSA_t;}
  ColorSA_t = record
    r,g,b,a : Fixed16_t;
  end;

{ ----------------------POINTS------------------------ }

  LPPointL_t = pointer; {^PointL_t;}
  PointL_t = record
    x,y,z : Fixed32_t;
  end;

  LPPointLW_t = pointer; {^PointLW_t;}
  PointLW_t = record
    x,y,z,w : Fixed32_t;
  end;

  LPPointF_t = pointer; {^PointF_t;}
  PointF_t = record
    x,y,z : Float_t;
  end;

  LPPointFW_t = pointer; {^PointFW_t;}
  PointFW_t = record
    x,y,z,w : Float_t;
  end;

  LPLineF_t = pointer; {^LineF_t;}
  LineF_t = record
    point, vector : PointF_t;
  end;

  LPPlaneF_t = pointer; {^PlaneF_t;}
  PlaneF_t = record
    a,b,c,d : Float_t;
  end;

  LPSphereF_t = pointer; {^SphereF_t;}
  SphereF_t = record
    center : PointF_t;
    r : Float_t;
  end;

  LPLineL_t = pointer; {^LineL_t;}
  LineL_t = record
    point, vector : PointL_t;
  end;

  LPPlaneL_t = pointer; {^PlaneL_t;}
  PlaneL_t = record
    a,b,c,d : Fixed32_t;
  end;

  LPSphereL_t = pointer; {^SphereL_t;}
  SphereL_t = record
    center : PointL_t;
    r : Fixed32_t;
  end;

  G3dCPlaneProp_t = (G3DCP_DATA, G3DCP_SIGN);

  G3dPrimitive_t = Word;

  G3dLightProp_t = (
		G3DL_DIMMER,
		G3DL_AMBIENT,
		G3DL_DIFFUSE,
		G3DL_SPECULAR,	
		G3DL_LOCATION,	
		G3DL_DIRECTION,	
		G3DL_SPOT_EXP,	
		G3DL_SPOT_ANGLE,
		G3DL_ATTENA,	
		G3DL_ATTENB,	
		G3DL_ATTENC	
  );

  G3dHazeMode_t = (
		G3DL_OFF,
		G3DL_LINEAR,
		G3DL_EXP,
		G3DL_EXP2
  );

  G3dMattProp_t = (
		G3DM_AMBIENT,	
		G3DM_DIFFUSE,	
		G3DM_SPECULAR,	
		G3DM_EMIT,	
		G3DM_SHINE,	
		G3DM_CULL,	
		G3DM_BACK_AMBIENT,
		G3DM_BACK_DIFFUSE,
		G3DM_BACK_SPECULAR,
		G3DM_BACK_EMIT,	
		G3DM_BACK_SHINE,
		G3DM_BACK_CULL,	
		G3DM_ALPHA,	
		G3DM_TEXTURE_HDL
  );

{ ----------------------TRANSFORMS------------------------ }

  G3dTransformStack_t = (
		G3DT_MODEL,
		G3DT_CAM_CLIP,
		G3DT_TEXTURE
  );

  TransformF_t = array[0..3,0..3] of Float_t;
  LPTransformF_t = pointer; {^TransformF_t;}
  Transform23F_t = array[0..1,0..2] of Float_t;
  LPTransform23F_t = pointer; {^Transform23F_t;}

{* ----------------------CAMERAS-------------------------- *}

  LPG3dProjection_t = pointer; {^G3dProjection_t;}
  G3dProjection_t = (
		G3DP_OFF,
		G3DP_PARALLEL,
		G3DP_PERSPECTIVE
  );


{ ----------SELECTION & MANIPULATION----------------------- }

  G3dTrackballEvent_t = (
		TB_START,
		TB_MOVE,
		TB_END
  );


const
{ 3DR constant definitions }

{ These are used by R3dPrimitive() Pick ONE of }

  R3D_PRM_POINTS    = 4;
  R3D_PRM_LINES     = 5;
  R3D_PRM_POLYLINE  = 6;
  R3D_PRM_TRIANGLES = 7;
  R3D_PRM_TRISTRIP  = 8;
  R3D_PRM_POLYGON   = 9;
  R3D_PRM_QUADS     = 10;
  R3D_PRM_QUADSTRIP = 11;

{  These are used by R3dGetState() and R3dSetState() Pick ONE of }
{  Enumerated states. Possible values mentioned in the comments
             section. Pick ONE of                                }

  R3D_STA_ENABLES            = 1; { use  R3D_SE_ }
  R3D_STA_DISABLES           = 2; { use  R3D_SE_ }
  R3D_STA_DRAW_BUFFER        = 3; { use  R3D_DB_ }
  R3D_STA_Z_FUNCTION         = 4; { use  R3D_ZF_ }
  R3D_STA_SRC_BLEND_FUNCTION = 5; { use  R3D_BF_ }
  R3D_STA_DST_BLEND_FUNCTION = 6; { use  R3D_BF_ }
  R3D_STA_TEX_FILTER         = 7; { use  R3D_TX_ }
  R3D_STA_TEX_FUNCTION       = 8; { use  R3D_TF_ }
  R3D_STA_ROP_FUNCTION       = 9; { use  R3D_RF_ }

{ Arbitary valued states.  Pick ONE of }

  R3D_STA_AREA_MASK      = 16;  { Area pattern bitmask }
  R3D_STA_LINE_MASK      = 48;  { Line pattern bitmask }
  R3D_STA_PIXEL_MASK     = 49;  { RGBA or bitmask      }
  R3D_STA_FG_COLOR       = 50;  { long, RGBA or index  }
  R3D_STA_BG_COLOR       = 51;  { long, RGBA or index  }
  R3D_STA_FG_DEPTH       = 52;  { FIXED 16.16 for raster}
  R3D_STA_BG_DEPTH       = 53;  { FIXED 16.16 for zclear}
  R3D_STA_TEX_LUMINANCE  = 54;  { >> Will Go away! }
  R3D_STA_TEX_ID         = 55;  { Handle for current texture.}
  R3D_STA_SCISSOR_LEFT   = 56;  
  R3D_STA_SCISSOR_TOP    = 57;  
  R3D_STA_SCISSOR_RIGHT  = 58;  { right,bottom, not inclusive }
  R3D_STA_SCISSOR_BOTTOM = 59;  { Validity Check done here.   }
  R3D_STA_ZOOM_X         = 60;
  R3D_STA_ZOOM_Y         = 61;

{ Possible values for: R3D_STA_ENABLES -- R3D_STA_DISABLES  Logical OR of }
 
  R3D_SE_SHADING            = $00000001; { interpolate r,g,b, a       }
  R3D_SE_TEXTURING          = $00000002; { interpolate u,v,w          }
  R3D_SE_MODULATION         = $00000004; { modulate color & texture   }
  R3D_SE_Z_BUFFERING        = $00000008; { interpolate z and compare  }
  R3D_SE_ANTIALIASING       = $00000010; { generate subpixel alpha    }

  R3D_SE_PIXEL_MASKING      = $00000020; { selective pixel update     }
  R3D_SE_Z_MASKING          = $00000040; { selective z update         }
  R3D_SE_PATTERNING         = $00000080; { selective pixel & z update }
  R3D_SE_SCISSORING         = $00000100; { selective pixel & z update }
  R3D_SE_CLIPPING           = $00000200; { selective pixel & z update }

  R3D_SE_BLENDING           = $00000400; { per-pixel operation        }
  R3D_SE_RASTEROP           = $00000800; { per-pixel operation        }
  R3D_SE_DITHERING          = $00001000; { per-pixel operation        }

  R3D_SE_FILTERING          = $00002000; { texture filter control     }
  R3D_SE_TRANSPARENCY       = $00004000; { texel use control          }
  R3D_SE_SWAP_WITH_ZOOM     = $00080000;
  R3D_SE_COLOR_INDEXED_MODE = $00200000;
  
{ Possible values for:  R3D_STA_DRAW_BUFFER   Pick one of }
    
  R3D_DB_FRONT       = 0; 
  R3D_DB_BACK        = 1;

{ Possible values for:  R3D_STA_Z_FUNCTION  Pick one of }

  R3D_ZF_NEVER  = 0;
  R3D_ZF_LT     = 1;
  R3D_ZF_EQ     = 2;
  R3D_ZF_LE     = 3;
  R3D_ZF_GT     = 4;
  R3D_ZF_NE     = 5;
  R3D_ZF_GE     = 6; 
  R3D_ZF_ALWAYS = 7; 

{ Possible values for:  R3D_STA_SRC_BLEND_FUNCTION
     R3D_STA_DST_BLEND_FUNCTION  Pick one of                   }
          
  R3D_BF_ZERO          = 0; 
  R3D_BF_ONE           = 1; 
  R3D_BF_SRC           = 2;  { Only  R3D_STA_DST_BLEND_FUNCTION }
  R3D_BF_SRC_INV       = 3;  { Only  R3D_STA_DST_BLEND_FUNCTION }
  R3D_BF_SRC_ALPHA     = 4; 
  R3D_BF_SRC_ALPHA_INV = 5; 
  R3D_BF_DST           = 6;  { Only  R3D_STA_SRC_BLEND_FUNCTION }
  R3D_BF_DST_INV       = 7;  { Only  R3D_STA_SRC_BLEND_FUNCTION }
  R3D_BF_DST_ALPHA     = 8; 
  R3D_BF_DST_ALPHA_INV = 9;

{ Possible values for:  R3D_STA_ROP_FUNCTION  Pick one of }

  R3D_RF_CLEAR       = 0; 
  R3D_RF_AND         = 1; 
  R3D_RF_AND_REVERSE = 2; 
  R3D_RF_COPY        = 3; 
  R3D_RF_AND_INVERT  = 4; 
  R3D_RF_NOOP        = 5; 
  R3D_RF_XOR         = 6; 
  R3D_RF_OR          = 7; 
  R3D_RF_NOR         = 8; 
  R3D_RF_EQUIV       = 9; 
  R3D_RF_INVERT      = 10;
  R3D_RF_OR_REVERSE  = 11; 
  R3D_RF_COPY_INVERT = 12; 
  R3D_RF_OR_INVERT   = 13; 
  R3D_RF_NAND        = 14; 
  R3D_RF_SET         = 15; 

{ Possible values for:  R3D_STA_TEX_FUNCTION  Pick ONE of }
      
  R3D_TF_REDUCE   = 1; 
  R3D_TF_MULTIPLY = 2; 
  R3D_TF_SHADE    = 3; 

{ Possible values for:  R3D_STA_TEX_FILTER  Pick ONE of }

  R3D_TX_BILINEAR         = 1;   { interpolate 4 pixels         }
  R3D_TX_CUSTOM           = 9;   { your favorite filter         }
  R3D_TX_MIPMAP_NEAREST   = 10;  { nearest mipmap               }
  R3D_TX_MIPMAP_LINEAR    = 11;  { interpolate between mipmaps  }
  R3D_TX_MIPMAP_BILINEAR  = 12;  { interpolate 4x within mipmap }
  R3D_TX_MIPMAP_TRILINEAR = 13;  { interpolate mipmaps,4 pixels }


{ Possible values for: RCInfo in the R3dCreateRC()
   prototype below Logical OR of                      }

  R3D_RI_DEFAULT       = $0000;
  R3D_RI_DOUBLE_BUFFER = $0001;
  R3D_RI_COLOR_INDEXED = $0002;
  R3D_RI_EXTENDED      = $0004;
  R3D_RI_RESERVED0     = $0008;
  R3D_RI_RESERVED1     = $0010;
  R3D_RI_RESERVED2     = $0020;
  R3D_RI_RESERVED3     = $0040;
  R3D_RI_RESERVED4     = $0080;
  R3D_RI_RESERVED5     = $0100;
  R3D_RI_RESERVED6     = $0200;
  R3D_RI_RESERVED7     = $0400;
  R3D_RI_RESERVED8     = $0800;

{ Possible values for: VtxInfo in the R3dPrimitive() prototype Logical OR }

  R3D_VI_FIXED   = $0000;
  R3D_VI_FLOAT   = $0001;
  R3D_VI_COLOR   = $0002;
  R3D_VI_TEXTURE = $0004;

{ Possible values for: BmpInfo in the R3dBitmap()
        prototype below  Logical OR of                    }
              
  R3D_BI_INDEX     = $0001;
  R3D_BI_CHROMAKEY = $0002;
  R3D_BI_ALPHA     = $0004;

{ Possible values for: SwapInfo in the R3dSwapBuffers()
        prototype below  Logical OR of                         }

  R3D_SI_SCISSOR = $0004;

{ Possible values for: ClearInfo in the R3dClearBuffers()
      prototype below  Logical OR of                          }

  R3D_CI_DRAW_BUFFER       = $0001;
  R3D_CI_ZBUFFER           = $0004;

{ Possible values for: TexInfo in the R3dNewTexture()
    prototype below.  Logical OR of                       }

  R3D_TI_DEFAULT   = $0000;
  R3D_TI_MIPMAP    = $0001;
  R3D_TI_HIRES     = $0002;
  R3D_TI_KEEP      = $0004;
  R3D_TI_INDEX     = $0010;
  R3D_TI_CHROMAKEY = $0020;
  R3D_TI_ALPHA     = $0040;

{ Maximum No of vertices to be passed using R3dPrimitive() }

  R3D_MAX_VERTICES  = 256;

{ 3DG constant definitions }

  ONE                   = 65536.0;
  ZONE                  = 268435455.0;
  COLONE                = 65535.0;
  MAX_C                 = 255;
  SMALL_NUMBER          = 1.0E-12;
  G3D_OFF               = 0;
  G3D_ON                = 1;

  G3D_MAX_VERTICES	= 256;
  G3D_MAX_LIST	        = 4096;
  G3D_MAX_PLANES	= Fixed32_t(8);
  G3D_TRANS_STACK_SIZE  = 32;

  G3D_PRM_NONE      = -1;
  G3D_PRM_POINTS    = R3D_PRM_POINTS;
  G3D_PRM_LINES     = R3D_PRM_LINES;
  G3D_PRM_POLYLINE  = R3D_PRM_POLYLINE;
  G3D_PRM_TRIANGLES = R3D_PRM_TRIANGLES;
  G3D_PRM_QUADS     = R3D_PRM_QUADS;
  G3D_PRM_TRISTRIP  = R3D_PRM_TRISTRIP;
  G3D_PRM_QUADSTRIP = R3D_PRM_QUADSTRIP;
  G3D_PRM_POLYGON   = R3D_PRM_POLYGON;

  G3D_MAX_LIGHTS    = Fixed32_t(16);
  G3D_MAX_MATTS	    = 16;


{ 3DR Documented Function Prototypes }


procedure R3dBitmap(hRC: R3dHandle_t; BmpInfo: Word_t; BmpBitsPixel: Word_t;
           BmpWidth: Word_t; BmpHeight: Word_t; BmpWidthBytes: Short_t;
           BmpSrcX: Short_t; BmpSrcY: Short_t; BmpDstX: Short_t;
           BmpDstY: Short_t; BmpData: Dword_t; BmpBuffer: LPByte_t);stdcall;

procedure R3dClearBuffers(hRC: R3dHandle_t; ClearInfo: Word_t);stdcall;

function  R3dCreateRC(hDC: R3dHandle_t; rcInfo: Word_t): Dword_t;stdcall;

function  R3dDeleteRC(hRC: R3dHandle_t): Dword_t;stdcall;

procedure R3dFreeTexture(hRC: R3dHandle_t; TexId: R3dHandle_t);stdcall;

function  R3dGetState(hRC: R3dHandle_t; State: Word_t): Dword_t;stdcall;

function  R3dLoadTexture(hRC: R3dHandle_t; TexInfo: Word_t;
           TexBitsPixel: Word_t; TexMipLevel: Word_t; TexWidth: Word_t;
           TexRows: Word_t; TexWidthBytes:  Short_t; TexId: Dword_t;
           TexData: Dword_t; TexBuffer: LPByte_t): Dword_t;stdcall;

function  R3dMessage(hRC: R3dHandle_t; InDataSize: Dword_t;
           InDataPtr: LPByte_t; OutDataSize: Dword_t;
           OutDataPtr: LPByte_t): Dword_t;stdcall;

function  R3dNewTexture(hRC: R3dHandle_t; TexInfo: Word_t;
           TexBitsPixel: Word_t; TexWidth: Word_t; TexHeight: Word_t): Dword_t;stdcall;

procedure R3dPrimitive(hRC: R3dHandle_t; PrmType: Word_t; VtxInfo: Word_t;
           VtxCount: Word_t; VtxSize: Word_t; VtxPtr: LPByte_t);stdcall;

procedure R3dSetState(hRC: R3dHandle_t; State: Word_t; Data: Dword_t);stdcall;

procedure R3dSwapBuffers(hRC: R3dHandle_t; SwapInfo: Word_t);stdcall;

{   3DR UnDocumented Function Prototypes }
{ The next are not in futures section but they are not documented either } 
procedure R3dApplyNewPalette(hRC: Dword_t);stdcall;

function  R3dSetPaletteEntry(hRC: Dword_t; PalEntry: Dword_t;
           Exactness: Long_t): Dword_t;stdcall;


{ 3DG Documented Function Prototypes }

procedure G3dSetState(hGC : G3dHandle_t; st : G3dState_t; data : LPFixed32_t);stdcall;

procedure G3dGetState(hGC : G3dHandle_t; st : G3dState_t; data : LPFixed32_t);stdcall;

function G3dCreateGC(hRC : R3dHandle_t) : G3dHandle_t;stdcall;

procedure G3dDeleteGC(hGC : G3dHandle_t);stdcall;

procedure G3dSetCoords(hGC : G3dHandle_t; coordinates : G3dCoords_t);stdcall;

procedure G3dQuerySystemCapabilities(hGC : G3dHandle_t; var cap : G3dCapabilities_t);stdcall;

procedure G3dGetClipPlane(hGC : G3dHandle_t; wCPPNum : Fixed32_t;
  prop : G3dCPlaneProp_t; data : LPFloat_t); stdcall;

procedure G3dSetClipPlane(hGC : G3dHandle_t; wCPPNum : Fixed32_t;
  prop : G3dCPlaneProp_t; data : LPFloat_t); stdcall;

procedure G3dQPatchF(hGC : G3dHandle_t; un : Fixed32_t; vn : Fixed32_t;
  p : LPPointFW_t; pstep : Fixed32_t; t : LPPointF_t; tstep : Fixed32_t;
  c : LPColorFA_t; cstep : Fixed32_t; NormFlag : longint); stdcall;

procedure G3dQPatchListF(hGC : G3dHandle_t; un : Fixed32_t; vn : Fixed32_t;
  qp : LPPointFW_t; qpstep : Fixed32_t; qt : LPPointF_t; qtstep : Fixed32_t;
  qc : LPColorFA_t; qcstep : Fixed32_t; NormFlag : longint;
  p : LPPointF_t; pstep : Fixed32_t; n : LPPointF_t; nstep : Fixed32_t;
  t : LPPointF_t; tstep : Fixed32_t;
  c : LPColorFA_t; cstep : Fixed32_t; idx : LPFixed16_t); stdcall;

procedure G3dQPatchL(hGC : G3dHandle_t; un : Fixed32_t; vn : Fixed32_t;
  p : LPPointLW_t; pstep : Fixed32_t; t : LPPointL_t; tstep : Fixed32_t;
  c : LPColorSA_t; cstep : Fixed32_t; NormFlag : longint); stdcall;

procedure G3dQPatchListL(hGC : G3dHandle_t; un : Fixed32_t; vn : Fixed32_t;
  qp : LPPointLW_t; qpstep : Fixed32_t; qt : LPPointL_t;
  qtstep : Fixed32_t;
  qc : LPColorSA_t; qcstep : Fixed32_t; NormFlag : longint;
  p : LPPointL_t; pstep : Fixed32_t; n : LPPointL_t;
  nstep : Fixed32_t; t : LPPointL_t; tstep : Fixed32_t;
  c : LPColorSA_t; cstep : Fixed32_t; idx : LPFixed16_t); stdcall;

procedure G3dPrimitiveF(hGC : G3dHandle_t; prim : G3dPrimitive_t; nvtx : Fixed32_t;
  p : LPPointF_t; pstep : Fixed32_t; n : LPPointF_t; nstep : Fixed32_t;
  t : LPPointF_t; tstep : Fixed32_t; c : LPColorFA_t; cstep : Fixed32_t); stdcall;

procedure G3dPrimitiveL(hGC : G3dHandle_t; prim : G3dPrimitive_t; nvtx : Fixed32_t;
  p : LPPointL_t; pstep : Fixed32_t; n : LPPointL_t; nstep : Fixed32_t;
  t : LPPointL_t; tstep : Fixed32_t; c : LPColorSA_t; cstep : Fixed32_t); stdcall;

procedure G3dBeginPrim(hGC : G3dHandle_t; prim : G3dPrimitive_t; nvtx : Fixed32_t); stdcall;

procedure G3dAddPrimVtxF(hGC : G3dHandle_t; p,n,t : LPPointF_t; c : LPColorFA_t); stdcall;

procedure G3dAddPrimVtxL(hGC : G3dHandle_t; p,n,t : LPPointL_t; c : LPColorSA_t); stdcall;

procedure G3dEndPrim(hGC : G3dHandle_t); stdcall;

procedure G3dPolygonListF(hGC : G3dHandle_t; npoly, tvtx : Fixed32_t;
  p : LPPointF_t; pstep : Fixed32_t; n : LPPointF_t; nstep : Fixed32_t;
  t : LPPointF_t; tstep : Fixed32_t; c : LPColorFA_t; cstep : Fixed32_t;
  polylist : LPFixed16_t); stdcall;

procedure G3dPolygonListL(hGC : G3dHandle_t; npoly, tvtx : Fixed32_t;
  p : LPPointL_t; pstep : Fixed32_t; n : LPPointL_t; nstep : Fixed32_t;
  t : LPPointL_t; tstep : Fixed32_t; c : LPColorSA_t; cstep : Fixed32_t;
  polylist : LPFixed16_t); stdcall;

procedure G3dPolygonListFF(hGC : G3dHandle_t; npoly, tvtx : Fixed32_t;
  p : LPPointF_t; pstep : Fixed32_t; n : LPPointF_t; nstep : Fixed32_t;
  fn : LPPlaneF_t; fnstep : Fixed32_t;
  t : LPPointF_t; tstep : Fixed32_t; c : LPColorFA_t; cstep : Fixed32_t;
  polylist : LPFixed16_t); stdcall;

procedure G3dPolygonListFL(hGC : G3dHandle_t; npoly, tvtx : Fixed32_t;
  p : LPPointL_t; pstep : Fixed32_t; n : LPPointL_t; nstep : Fixed32_t;
  fn : LPPlaneL_t; fnstep : Fixed32_t;
  t : LPPointL_t; tstep : Fixed32_t; c : LPColorSA_t; cstep : Fixed32_t;
  polylist : LPFixed16_t); stdcall;

procedure G3dPrimitiveTF(hGC : G3dHandle_t; nvtx : Fixed32_t;
  p : LPPointF_t; pstep : Fixed32_t; n : LPPointF_t; nstep : Fixed32_t;
  t : LPPointF_t; tstep : Fixed32_t; c : LPColorFA_t; cstep : Fixed32_t); stdcall;

procedure G3dPrimitiveTL(hGC : G3dHandle_t; nvtx : Fixed32_t;
  p : LPPointL_t; pstep : Fixed32_t; n : LPPointL_t; nstep : Fixed32_t;
  t : LPPointL_t; tstep : Fixed32_t; c : LPColorSA_t; cstep : Fixed32_t); stdcall;

procedure G3dResetLight(hGC : G3dHandle_t; wLightNum : Fixed32_t); stdcall;

procedure G3dGetLight(hGC : G3dHandle_t; wLightNum : Fixed32_t;
  prop: G3dLightProp_t; data : LPFloat_t); stdcall;

procedure G3dSetLight(hGC : G3dHandle_t; wLightNum : Fixed32_t;
  prop: G3dLightProp_t; data : LPFloat_t); stdcall;

procedure G3dSetHazeColor(hGC : G3dHandle_t; color : LPColorF_t); stdcall;

procedure G3dSetHazeFunction(hGC : G3dHandle_t; mode : G3dHazeMode_t;
  density, hnear, hfar : Float_t); stdcall;

procedure G3dGetMatt(hGC : G3dHandle_t; wMattNum : Fixed32_t;
  prop : G3dMattProp_t; data : LPFloat_t); stdcall;

procedure G3dSetMatt(hGC : G3dHandle_t; wMattNum : Fixed32_t;
  prop : G3dMattProp_t; data : LPFloat_t); stdcall;

function G3dGetCurrentMatt(hGC : G3dHandle_t):Fixed32_t; stdcall;

procedure G3dSetCurrentMatt(hGC : G3dHandle_t; wMattNum : Fixed32_t); stdcall;

procedure G3dSetActiveStack(hGC : G3dHandle_t; stack : G3dTransformStack_t); stdcall;

procedure G3dClearStack(hGC : G3dHandle_t); stdcall;

procedure G3dPushTransform(hGC : G3dHandle_t); stdcall;

procedure G3dPopTransform(hGC : G3dHandle_t); stdcall;

procedure G3dIdentityTransform(hGC : G3dHandle_t); stdcall;

procedure G3dRotateTransform(hGC : G3dHandle_t; theta, x, y, z : Float_t); stdcall;

procedure G3dTranslateTransform(hGC : G3dHandle_t; tx, ty, tz : Float_t); stdcall;

procedure G3dScaleTransform(hGC : G3dHandle_t; sx, sy, sz : Float_t); stdcall;

procedure G3dPreMultTransform(hGC : G3dHandle_t; mat : LPTransformF_t); stdcall;

procedure G3dPostMultTransform(hGC : G3dHandle_t; mat : LPTransformF_t); stdcall;

procedure G3dSetTransform(hGC : G3dHandle_t; mat : LPTransformF_t); stdcall;

procedure G3dGetTransform(hGC : G3dHandle_t; mat : LPTransformF_t); stdcall;

procedure G3dTransposeMatrix(mat : LPTransformF_t); stdcall;

procedure G3dInvertMatrix(src : LPTransformF_t; dest : LPTransformF_t); stdcall;

procedure G3dMultMatrix(m0, m1, m2 : LPTransformF_t); stdcall;

procedure G3dCopyMatrix(m0, m1 : LPTransformF_t); stdcall;

procedure G3dRotateMatrix(theta, a, b, c : Float_t; mat : LPTransformF_t); stdcall;

procedure G3dIdentityMatrix(mat : LPTransformF_t); stdcall;

function G3dUnitVector(vec : LPPointF_t) : Float_t; stdcall;

procedure G3dUnitVector2(vin, vout : LPPointF_t); stdcall;

function G3dluDecomp(amat : pointer; N : Fixed32_t;
  index : pointer; ric : Float_t) : Fixed32_t; stdcall;

procedure G3dluBackSub(amat : pointer; N : Fixed32_t;
  index : pointer; b: Float_t); stdcall;

procedure G3dSolveLinearEqns(amat : pointer; b : pointer; N : Fixed32_t); stdcall;

procedure G3dTransformPointF(ptin : LPPointF_t;
  ptout : LPPointFW_t; mat : LPTransformF_t); stdcall;

procedure G3dTransformNormalF(pin : LPPointF_t; pout : LPPointF_t;
  mat : LPTransformF_t); stdcall;

function G3dGetModelCamMatrix(hGC : G3dHandle_t) : LPTransformF_t; stdcall;

function G3dGetInverseModelCamMatrix(hGC : G3dHandle_t) : LPTransformF_t; stdcall;

procedure G3dTranslateMatrix(x, y, z : Float_t; mat : LPTransformF_t); stdcall;

procedure G3dScaleMatrix(x, y, z : Float_t; mat : LPTransformF_t); stdcall;

procedure G3dGetCameraProjection(hGC : G3dHandle_t; proj : LPG3dProjection_t); stdcall;

procedure G3dGetCameraPosition(hGC : G3dHandle_t; loc : LPPointFW_t;
  dir, up : LPPointF_t); stdcall;

procedure G3dGetCameraView(hGC : G3dHandle_t;
  right, left, top, bottom, pnear, pfar : LPFloat_t); stdcall;

procedure G3dGetCameraPort(hGC : G3dHandle_t;
  right, left, top, bottom : LPFloat_t); stdcall;

procedure G3dSetCameraProjection(hGC : G3dHandle_t; proj : G3dProjection_t); stdcall;

procedure G3dSetCameraPosition(hGC : G3dHandle_t; loc : LPPointFW_t;
  dir, up : LPPointF_t); stdcall;

procedure G3dSetCameraView(hGC : G3dHandle_t;
  right, left, top, bottom, pnear, pfar : Float_t); stdcall;

procedure G3dSetCameraPort(hGC : G3dHandle_t;
  right, left, top, bottom : Float_t); stdcall;

procedure G3dSetWindowSize(hGC : G3dHandle_t; ww, wh : Float_t); stdcall;

procedure G3dTrackball(event:G3dTrackballEvent_t; center:LPPointF_t; radius,x,y:Float_t;
  axis:LPPointF_t; theta:LPFloat_t); stdcall;

procedure G3dPointWindowToObject(hGC:G3dHandle_t; x,y:Float_t; line:LPLineF_t); stdcall;

procedure G3dLineWindowToObject(hGC:G3dHandle_t; x0,y0,x1,y1:Float_t; plane:LPPlaneF_t); stdcall;

procedure G3dVectorWindowToObject(hGC:G3dHandle_t; x0,y0,x1,y1:Float_t; vec:LPLineF_t); stdcall;

procedure G3d3PointPlane(a, b, c:LPPointF_t; plane:LPPlaneF_t); stdcall;

procedure G3dPointNormalPlane(p, n:LPPointF_t; plane:LPPlaneF_t); stdcall;

procedure G3dBoundingSphere(npts:Fixed32_t; pts:LPPointF_t; sphere:LPSphereF_t); stdcall;

function G3dLinePlaneIntersect(line:LPLineF_t; plane:LPPlaneF_t; vtx:LPPointF_t):Fixed32_t; stdcall;

function G3dLineSphereIntersect(line:LPLineF_t; sphere:LPSphereF_t; vtx:LPPointF_t):Fixed32_t; stdcall;


{ ---------------------- UTILITIES ------------------------------- }

function G3dqdsqrt(val : Float_t) : Float_t; stdcall;     { quick and dirty square root }

function G3dqdpow(x : Float_t; exp : Fixed32_t) : Float_t; stdcall;{ quick and dirty power }

function G3dqdsin(val : Float_t) : Float_t; stdcall;      { quick and dirty sin }

function G3dqdcos(val : Float_t) : Float_t; stdcall;      { quick and dirty cos }

function G3dVecLengthFloat(x,y,z : Float_t) : Float_t;

function G3dVecLengthFixed(x,y,z : Fixed32_t) : Float_t;

function G3dVecLength(x,y,z : Float_t) : Float_t;

function G3dVecLengthPt(p:LPPointF_t):Float_t;

procedure G3dCross( A, B:PointF_t; C:LPPointF_t );

function G3dDot(a,b:PointF_t):Float_t;

{ ---------------------- IMAGES ------------------------------- }

function G3dReadBMPFile(ptrToFilename : PChar; dibHandle : THandle) : THandle; stdcall;

function G3dLoadTexture(hGC : G3dHandle_t; ptrToFilename : PChar) : R3dHandle_t; stdcall;

function G3dCreateTexture(hGC : G3dHandle_t; dibHandle : THandle) : R3dHandle_t; stdcall;

procedure G3dTransformBMP(bmp1 : THandle; bmp2 : PHandle;
  xform : LPTransform23F_t; x0, y0, w, h, bpp, rgb : Fixed32_t); stdcall;


implementation

{ 3DR implementation details }

function  R3dSetPaletteEntry;  external '3DR32.DLL';
procedure R3dApplyNewPalette;  external '3DR32.DLL';
function  R3dMessage;          external '3DR32.DLL';
function  R3dGetState;         external '3DR32.DLL';
function  R3dNewTexture;       external '3DR32.DLL';
function  R3dLoadTexture;      external '3DR32.DLL';
procedure R3dFreeTexture;      external '3DR32.DLL';
procedure R3dBitmap;           external '3DR32.DLL';
function  R3dCreateRC;         external '3DR32.DLL';
function  R3dDeleteRC;         external '3DR32.DLL';
procedure R3dPrimitive;        external '3DR32.DLL';
procedure R3dSetState;         external '3DR32.DLL';
procedure R3dSwapBuffers;      external '3DR32.DLL';
procedure R3dClearBuffers;     external '3DR32.DLL';

{ 3DG implementation details }

function G3dVecLengthFloat(x,y,z : Float_t) : Float_t;
begin
  Result := sqrt(sqr(x)+sqr(y)+sqr(z));
end;

function G3dVecLength(x,y,z : Float_t) : Float_t;
begin
  Result := sqrt(sqr(x)+sqr(y)+sqr(z));
end;

function G3dVecLengthFixed(x,y,z : Fixed32_t) : Float_t;
begin
  Result := sqrt(sqr(x)+sqr(y)+sqr(z));
end;

function G3dVecLengthPt(p:LPPointF_t):Float_t;
type
  PPointF_t = ^PointF_t;
var
  p1 : PPointF_t;
begin
  p1:=p;
  with p1^ do Result:=sqrt(x*x + y*y + z*z);
end;

procedure G3dCross( A, B:PointF_t; C:LPPointF_t );
type
  PPointF_t = ^PointF_t;
var
  C1 : PPointF_t;
begin
  C1:=C;
  C1^.x :=  A.y * B.z - A.z * B.y;
  C1^.y := -A.x * B.z + A.z * B.x;
  C1^.z :=  A.x * B.y - A.y * B.x;
end;

function G3dDot(a,b:PointF_t):Float_t;
begin
  Result := a.x*b.x+a.y*b.y+a.z*b.z;
end;

procedure G3dSetState;  external '3DG32.DLL';
procedure G3dGetState;  external '3DG32.DLL';
function G3dCreateGC;  external '3DG32.DLL';
procedure G3dDeleteGC;  external '3DG32.DLL';
procedure G3dSetCoords;  external '3DG32.DLL';
procedure G3dQuerySystemCapabilities;  external '3DG32.DLL';
procedure G3dGetClipPlane;  external '3DG32.DLL';
procedure G3dSetClipPlane;  external '3DG32.DLL';
procedure G3dQPatchF;  external '3DG32.DLL';
procedure G3dQPatchListF;  external '3DG32.DLL';
procedure G3dQPatchL;  external '3DG32.DLL';
procedure G3dQPatchListL;  external '3DG32.DLL';
procedure G3dPrimitiveF;  external '3DG32.DLL';
procedure G3dPrimitiveL;  external '3DG32.DLL';
procedure G3dPrimitiveTF;  external '3DG32.DLL';
procedure G3dPrimitiveTL;  external '3DG32.DLL';
procedure G3dBeginPrim;  external '3DG32.DLL';
procedure G3dAddPrimVtxF;  external '3DG32.DLL';
procedure G3dAddPrimVtxL;  external '3DG32.DLL';
procedure G3dEndPrim;  external '3DG32.DLL';
procedure G3dPolygonListF;  external '3DG32.DLL';
procedure G3dPolygonListL;  external '3DG32.DLL';
procedure G3dPolygonListFF;  external '3DG32.DLL';
procedure G3dPolygonListFL;  external '3DG32.DLL';
procedure G3dResetLight;  external '3DG32.DLL';
procedure G3dGetLight;  external '3DG32.DLL';
procedure G3dSetLight;  external '3DG32.DLL';
procedure G3dSetHazeColor;  external '3DG32.DLL';
procedure G3dSetHazeFunction;  external '3DG32.DLL';
procedure G3dGetMatt;  external '3DG32.DLL';
procedure G3dSetMatt;  external '3DG32.DLL';
procedure G3dSetCurrentMatt;  external '3DG32.DLL';
function G3dGetCurrentMatt;  external '3DG32.DLL';
procedure G3dSetActiveStack;  external '3DG32.DLL';
procedure G3dClearStack;  external '3DG32.DLL';
procedure G3dPushTransform;  external '3DG32.DLL';
procedure G3dPopTransform;  external '3DG32.DLL';
procedure G3dIdentityTransform;  external '3DG32.DLL';
procedure G3dRotateTransform;  external '3DG32.DLL';
procedure G3dTranslateTransform;  external '3DG32.DLL';
procedure G3dScaleTransform;  external '3DG32.DLL';
procedure G3dPreMultTransform;  external '3DG32.DLL';
procedure G3dPostMultTransform;  external '3DG32.DLL';
procedure G3dSetTransform;  external '3DG32.DLL';
procedure G3dGetTransform;  external '3DG32.DLL';
procedure G3dTransposeMatrix;  external '3DG32.DLL';
procedure G3dInvertMatrix;  external '3DG32.DLL';
procedure G3dMultMatrix;  external '3DG32.DLL';
procedure G3dCopyMatrix;  external '3DG32.DLL';
procedure G3dRotateMatrix;  external '3DG32.DLL';
procedure G3dIdentityMatrix;  external '3DG32.DLL';
function G3dUnitVector;  external '3DG32.DLL';
procedure G3dUnitVector2;  external '3DG32.DLL';
function G3dluDecomp;  external '3DG32.DLL';
procedure G3dluBackSub;  external '3DG32.DLL';
procedure G3dSolveLinearEqns;  external '3DG32.DLL';
procedure G3dTransformPointF;  external '3DG32.DLL';
procedure G3dTransformNormalF;  external '3DG32.DLL';
function G3dGetModelCamMatrix;  external '3DG32.DLL';
function G3dGetInverseModelCamMatrix;  external '3DG32.DLL';
procedure G3dTranslateMatrix;  external '3DG32.DLL';
procedure G3dScaleMatrix;  external '3DG32.DLL';
procedure G3dGetCameraProjection;  external '3DG32.DLL';
procedure G3dGetCameraPosition;  external '3DG32.DLL';
procedure G3dGetCameraView;  external '3DG32.DLL';
procedure G3dGetCameraPort;  external '3DG32.DLL'; 
procedure G3dSetCameraProjection;  external '3DG32.DLL';
procedure G3dSetCameraPosition;  external '3DG32.DLL';
procedure G3dSetCameraView;  external '3DG32.DLL';
procedure G3dSetCameraPort;  external '3DG32.DLL'; 
procedure G3dSetWindowSize;  external '3DG32.DLL'; 
procedure G3dTrackball;  external '3DG32.DLL';
procedure G3dPointWindowToObject;  external '3DG32.DLL';
procedure G3dLineWindowToObject;  external '3DG32.DLL'; 
procedure G3dVectorWindowToObject;  external '3DG32.DLL';
procedure G3d3PointPlane;  external '3DG32.DLL';
procedure G3dPointNormalPlane;  external '3DG32.DLL';
procedure G3dBoundingSphere;  external '3DG32.DLL';
function G3dLinePlaneIntersect;  external '3DG32.DLL'; 
function G3dLineSphereIntersect;  external '3DG32.DLL'; 
function G3dqdsqrt;  external '3DG32.DLL'; 
function G3dqdpow;  external '3DG32.DLL'; 
function G3dqdsin;  external '3DG32.DLL'; 
function G3dqdcos;  external '3DG32.DLL'; 
function G3dReadBMPFile;  external '3DG32.DLL'; 
function G3dLoadTexture;  external '3DG32.DLL'; 
function G3dCreateTexture;  external '3DG32.DLL'; 
procedure G3dTransformBMP;  external '3DG32.DLL';

end.
